﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.ActivationInfo;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.ActivatioInfo;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.Store;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class ActivationService : IActivationInfoService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public ActivationService(ISqlSugarClient db)
        {
            _db = db;
        }

        /// <summary>
        /// 获取授权码分页
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetActivationInfoPage(ActivationInfoQueryDto dto)
        {
            RefAsync<int> total = 0;
            var deviceList = await _db.Queryable<BPA_DeviceInfo>().ToListAsync();
            var logList = await _db.Queryable<BPA_ActivationLog>().ToListAsync();
            var companyList = await _db.Queryable<BPA_Company>().ToListAsync();
            var res = await _db.Queryable<BPA_ActivationInfo>().LeftJoin<BPA_DeviceInfo>((a, b) => a.AutoKey == b.AutoKey)
                .ClearFilter()
                .WhereIF(dto.Status != null, (a, b) => a.Status == dto.Status)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.DeviceName), (a, b) => b.Name.Contains(dto.DeviceName))
                .OrderByDescending((a, b) => a.CreateAt)
                .Select((a, b) => new ActivationInfoDto
                {
                    Id = a.Id,
                    OnlineKey = a.OnlineKey,
                    OfflineKey = a.OfflineKey,
                    EffectiveDate = a.EffectiveDate,
                    Status = a.Status,
                    AutoKey = a.AutoKey,
                    DeviceName = b.Name,
                    Log = new ActivationLogDto()
                }).Mapper(a =>
                {
                    var log = logList.FirstOrDefault(t => t.ActivationId == a.Id);
                    if (log != null)
                    {
                        a.Log = new ActivationLogDto
                        {
                            Id = log.Id,
                            ActivationId = log.ActivationId,
                            OnlineKey = log.OnlineKey,
                            OfflineKey = log.OfflineKey,
                            ActivationDate = log.ActivationDate,
                            AutoKey = log.AutoKey,
                            DeviceName = deviceList.FirstOrDefault(t => t.AutoKey == log.AutoKey)?.Name,
                            Company = companyList.FirstOrDefault(t => t.Id == log.GroupId)?.Name
                        };
                    }
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            var util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }

        /// <summary>
        /// 更新授权码
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateActivationInfo(ActivationInfoUpDto dto)
        {
            var activationInfoList = await _db.Queryable<BPA_ActivationInfo>().ToListAsync();
            string onlinekey;
            if (dto.EffectiveDate != null)
                onlinekey = ConvertString(dto.EffectiveDate.Value.ToString("yyyy-MM-dd"), 0);
            else
                onlinekey = ConvertString(DateTime.Now.AddYears(50).ToString("yyyy-MM-dd"), 0);
            if (string.IsNullOrWhiteSpace(dto.Id))
            {
                var authInfo = new BPA_ActivationInfo
                {
                    OnlineKey = onlinekey,
                    EffectiveDate = dto.EffectiveDate != null ? dto.EffectiveDate.Value.Date : DateTime.Now.AddYears(50).Date,
                    Status = 1
                };
                await _db.Insertable(authInfo).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                var authInfo = activationInfoList.FirstOrDefault(t => t.Id == dto.Id);
                if (authInfo == null)
                    throw Oops.Oh("授权码不存在或已删除");
                authInfo.EffectiveDate = dto.EffectiveDate != null ? dto.EffectiveDate.Value.Date : DateTime.Now.AddYears(50).Date;
                await _db.Updateable(authInfo).ExecuteCommandAsync();
            }
            return true;
        }

        /// <summary>
        /// 生成离线授权码
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<string> UpdateOfflineKey(OfflineKeyDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var activation = await _db.Queryable<BPA_ActivationInfo>().FirstAsync(t => t.Id == dto.Id);
                if (activation == null)
                    throw Oops.Oh("激活码不存在或已删除");
                if (activation.Status != 1)
                    throw Oops.Oh("激活码已被激活");
                activation.OfflineKey = ConvertString(activation.EffectiveDate.ToString("yyyy-MM-dd") + "|" + dto.AutoKey.ToString("D3"), 1);
                activation.AutoKey = dto.AutoKey;
                await _db.Updateable(activation).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return activation.OfflineKey;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除授权码
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteActivationInfo(string id)
        {
            var activationInfo = await _db.Queryable<BPA_ActivationInfo>().FirstAsync(t => t.Id == id);
            if (activationInfo == null)
                throw Oops.Oh("该激活码不存在或已删除");
            await _db.Deleteable(activationInfo).ExecuteCommandAsync();
            return true;
        }

        public static string ConvertString(string input, int type)
        {
            var random = new Random();
            var a = (char)random.Next('A', 'Z' + 1);
            var b = (char)random.Next('A', 'Z' + 1);
            var mapping = new Dictionary<char, char>
            {
                { '1', 'U' },
                { '2', 'J' },
                { '3', 'C' },
                { '4', 'B' },
                { '5', 'L' },
                { '6', 'P' },
                { '7', 'A' },
                { '8', 'F' },
                { '9', 'G' },
                { '0', 'E' },
                { '-', 'R' },
                { '|', 'Q' }
            };
            var result = new StringBuilder();
            result.Append(a);
            result.Append(b);
            foreach (char i in input)
            {
                result.Append(mapping[i]);
            }
            int count;
            if (type == 1)
                count = 16 - result.Length;
            else
                count = 14 - result.Length;
            for (int i = 0; i < count; i++)
            {
                result.Append((char)random.Next('A', 'Z' + 1));
            }
            return result.ToString();
        }
    }
}
