﻿using BPA.Aliyun.OSS;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Interface;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.ActivatioInfo;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.Recipe;
using BPA.SAAS.Manage.Core.RecipeAttribute;
using BPA.SAAS.Manage.Core.Store;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class InterfaceService : IInterfaceService, ITransient
    {
        private readonly ISqlSugarClient _db;
        private AliyunOssClient _aliyunOssClient;
        private readonly IHttpContextAccessor _httpContextAccessor;
        public InterfaceService(ISqlSugarClient db, AliyunOssClient aliyunOssClient, IHttpContextAccessor httpContextAccessor)
        {
            _db = db;
            _aliyunOssClient = aliyunOssClient;
            _httpContextAccessor = httpContextAccessor;
        }

        private int GetAutoKeyFromHeaders()
        {
            var headers = _httpContextAccessor.HttpContext?.Request.Headers;
            if (headers != null && headers.TryGetValue("autoKey", out var autoKey))
            {
                if (int.TryParse(autoKey, out var result))
                    return result;
            }
            return 0;
        }

        #region 获取信息
        #region 账号
        /// <summary>
        /// 获取所有账号信息
        /// </summary>
        /// <returns></returns>
        public async Task<List<UsersDto>> GetUsers()
        {
            return await _db.Queryable<BPA_Users>().ClearFilter().LeftJoin<BPA_Company>((a, b) => a.GroupId == b.Id)
                .OrderByDescending((a, b) => a.CreateAt)
                .Select((a, b) => new UsersDto
                {
                    Id = a.Id,
                    Name = a.Name,
                    Account = a.Account,
                    Password = a.Password,
                    Phone = a.Phone,
                    AdminType = a.AdminType,
                    SysOrgId = a.OrgId,
                    CompanyName = b.Name,
                    CreateAt = a.CreateAt
                }).ToListAsync();
        }
        #endregion

        #region 设备autokey获取信息
        /// <summary>
        /// 获取设备信息
        /// </summary>
        /// <returns></returns>
        public async Task<DeviceInfoDto> GetDeviceInfo()
        {
            var autoKey = GetAutoKeyFromHeaders();
            var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.AutoKey == autoKey);
            var store = await _db.Queryable<BPA_Store>().FirstAsync(t => t.Id == device.StoreId);
            var organize = await _db.Queryable<BPA_Organize>().FirstAsync(t => t.Id == store.OrgId);
            return new DeviceInfoDto
            {
                StoreId = store.Id,
                StoreName = store.Name,
                StorePhone = store.Phone,
                DeviceId = device.Id,
                DeviceName = device.Name,
                DeviceAutokey = device.AutoKey.ToString(),
                DevicePosition = store.Position,
                OrgName = organize?.Name,
                ContactPhone = "13566666666"
            };
        }
        #endregion

        #region 获取配方信息
        /// <summary>
        /// 获取所有配方信息
        /// </summary>
        /// <returns></returns>
        public async Task<RecipeDto> GetRecipeList()
        {
            var result = new RecipeDto
            {
                RecipeGroupList = new List<BPA_RecipeGroup>(),
                RecipeTemplateList = new List<BPA_RecipeTemplate>(),
                RecipeInfoList = new List<BPA_RecipeInfo>(),
                RecipeVersionList = new List<BPA_RecipeVersion>(),
                RecipeAttrList = new List<BPA_RecipeAttribute>(),
                RecipeAttrValueList = new List<BPA_RecipeAttributeValue>(),
                RecipeBaseConfigList = new List<BPA_RecipeBaseConfig>(),
                RecipeAttrConfigList = new List<BPA_RecipeConfig>(),
                RecipeConfigDetailList = new List<BPA_RecipeConfigDetail>(),
            };
            var autoKey = GetAutoKeyFromHeaders();
            var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync();
            if (device == null)
                return result;
            result.RecipeGroupList = await _db.Queryable<BPA_RecipeGroup>().Where(t => t.AutoKey == autoKey).OrderBy(a => a.IsWeight).OrderBy(a => a.Sort).ToListAsync();
            result.RecipeTemplateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == autoKey).OrderBy(a => a.RecipeGroupId).OrderBy(a => a.Sort).ToListAsync();
            result.RecipeTemplateList.ForEach(t =>
            {
                if (t.RecipeId == null)
                    t.RecipeId = "";
                if (t.VersionId == null)
                    t.VersionId = "";
            });
            result.RecipeInfoList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == autoKey).OrderByDescending(t => t.CreateAt).ToListAsync();
            var versionIdList = result.RecipeTemplateList.Select(t => t.VersionId).ToList();
            result.RecipeVersionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => t.AutoKey == autoKey && versionIdList.Contains(t.Id)).ToListAsync();
            result.RecipeAttrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => t.AutoKey == autoKey && versionIdList.Contains(t.RecipeVersionId)).OrderBy(t => t.RecipeVersionId).OrderBy(t => t.Sort).ToListAsync();
            result.RecipeAttrValueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => t.AutoKey == autoKey && result.RecipeAttrList.Select(a => a.Id).Contains(t.AttributeId)).OrderBy(t => t.AttributeId).OrderBy(t => t.Sort).ToListAsync();
            result.RecipeBaseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => t.AutoKey == autoKey && versionIdList.Contains(t.RecipeVersionId)).ToListAsync();
            result.RecipeAttrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.AutoKey == autoKey && versionIdList.Contains(t.RecipeVersionId)).ToListAsync();
            result.RecipeConfigDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => t.AutoKey == autoKey && (result.RecipeBaseConfigList.Select(a => a.Id).Contains(t.ConfigId) || result.RecipeAttrConfigList.Select(a => a.Id).Contains(t.ConfigId))).ToListAsync();
            return result;
        }
        #endregion
        #endregion

        #region 物料接口
        /// <summary>
        /// 获取所有物料列表
        /// </summary>
        /// <returns></returns>
        public async Task<List<BatchingDto>> GetBatching()
        {
            return await _db.Queryable<BPA_Batching>().Select(t => new BatchingDto
            {
                Id = t.Id,
                Code = t.Code,
                Name = t.Name,
                CreateAt = t.CreateAt,
                StockUint = t.StockUint,
                Type = t.Type,
                Key = t.Key
            }).ToListAsync();
        }

        /// <summary>
        /// 上传更新物料信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadBatching(List<BatchingDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
                var batchingAddList = new List<BPA_Batching>();
                var batchingUpList = new List<BPA_Batching>();
                foreach (var item in dto)
                {
                    var repeatData = batchingList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if (repeatData != null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "物料上传失败：重复物料数据", Id = item.Id, Name = item.Name });
                        continue;
                    }
                    var bathingData = batchingList.FirstOrDefault(t => t.Id == item.Id);
                    if (bathingData == null)
                    {
                        var code = GenerateRandomString();
                        while (batchingList.Any(t => t.Code == code))
                        {
                            code = GenerateRandomString();
                        }
                        batchingAddList.Add(new BPA_Batching
                        {
                            Id = item.Id,
                            Name = item.Name,
                            Code = code,
                            StockUint = item.StockUint,
                            Type = item.Type,
                            Key = item.Key
                        });
                    }
                    else
                    {
                        bathingData.Name = item.Name;
                        bathingData.StockUint = item.StockUint;
                        bathingData.Type = item.Type;
                        bathingData.Key = item.Key;
                        batchingUpList.Add(bathingData);
                    }
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "物料上传成功", Id = item.Id, Name = item.Name });
                }
                if (errorList.Count > 0)
                    return errorList;
                if (batchingAddList.Count > 0)
                    await _db.Insertable(batchingAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                await _db.Updateable(batchingUpList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        private static readonly Random random = new();
        public static string GenerateRandomString()
        {
            const string letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
            const string digits = "0123456789";
            StringBuilder result = new();
            for (int i = 0; i < 4; i++)
            {
                result.Append(letters[random.Next(letters.Length)]);
            }
            for (int i = 0; i < 4; i++)
            {
                result.Append(digits[random.Next(digits.Length)]);
            }
            return result.ToString();
        }

        /// <summary>
        /// 删除物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> DeleteBatching(List<BatchingDelDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
                var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().ToListAsync();
                var batchingDelList = new List<BPA_Batching>();
                foreach (var item in dto)
                {
                    var batching = batchingList.FirstOrDefault(t => t.Id == item.Id);
                    if (batching == null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "物料删除失败：未找到物料数据", Id = item.Id, Name = item.Name });
                        continue;
                    }
                    var isHave = detailList.Any(t => t.BatchingId == item.Id);
                    if (isHave)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "物料删除失败：该物料已被使用", Id = item.Id, Name = item.Name });
                        continue;
                    }
                    batchingDelList.Add(batching);
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "物料添删除成功", Id = item.Id, Name = item.Name });
                }
                if (errorList.Count > 0)
                    return errorList;
                await _db.Deleteable(batchingDelList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 配方接口

        #region 配方信息
        ///// <summary>
        ///// 获取分组下配方信息
        ///// </summary>
        ///// <param name="recipeGroupId"></param>
        ///// <returns></returns>
        //public async Task<List<RecipeInfoDto>> GetRecipeInfo(string recipeGroupId)
        //{
        //    //return await _db.Queryable<BPA_RecipeInfo>().Where(t => t.RecipeGroupId == recipeGroupId).OrderBy(a => a.Sort).Select(t => new RecipeInfoDto
        //    //{
        //    //    Id = t.Id,
        //    //    RecipeGroupId = t.RecipeGroupId,
        //    //    Name = t.Name,
        //    //    ReferenceWeight = t.ReferenceWeight,
        //    //    Sort = t.Sort
        //    //}).ToListAsync();
        //    return new List<RecipeInfoDto>();
        //}

        /// <summary>
        /// 上传更新配方信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeInfo(List<RecipeInfoDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var recipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var recipeAddList = new List<BPA_RecipeInfo>();
                var recipeUpList = new List<BPA_RecipeInfo>();
                foreach (var item in dto)
                {
                    var repeatData = recipeList.FirstOrDefault(t => t.Name == item.Name && t.Id != item.Id);
                    if (repeatData != null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "配方上传失败：重复配方数据", Id = item.Id, Name = item.Name });
                        continue;
                    }
                    var recipeData = recipeList.FirstOrDefault(t => t.Id == item.Id);
                    if (recipeData == null)
                    {
                        recipeAddList.Add(new BPA_RecipeInfo
                        {
                            Id = item.Id,
                            Name = item.Name,
                            IsWeight = item.IsWeight,
                            ReferenceWeight = item.ReferenceWeight,
                            AutoKey = autoKey
                        });
                    }
                    else
                    {
                        recipeData.Name = item.Name;
                        recipeData.IsWeight = item.IsWeight;
                        recipeData.ReferenceWeight = item.ReferenceWeight;
                        recipeUpList.Add(recipeData);
                    }
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方上传成功", Id = item.Id, Name = item.Name });
                }
                if (errorList.Count > 0)
                    return errorList;
                if (recipeAddList.Count > 0)
                    await _db.Insertable(recipeAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                await _db.Updateable(recipeUpList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        ///// <summary>
        ///// 删除配方
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeInfo(List<RecipeInfoDelDto> dto)
        //{
        //    try
        //    {
        //        _db.Ado.BeginTran();
        //        var errorList = new List<InterfaceMessageDto>();
        //        var successList = new List<InterfaceMessageDto>();
        //        var recipeList = await _db.Queryable<BPA_RecipeInfo>().ToListAsync();
        //        var relationList = await _db.Queryable<BPA_RecipeDeviceRelation>().ToListAsync();
        //        var attrList = await _db.Queryable<BPA_RecipeAttribute>().ToListAsync();
        //        var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
        //        var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().ToListAsync();
        //        var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().ToListAsync();
        //        var detailList = await _db.Queryable<BPA_RecipeConfigDetail>().ToListAsync();
        //        var recipeUpList = new List<BPA_RecipeInfo>();
        //        var attrDelList = new List<BPA_RecipeAttribute>();
        //        var valueDelList = new List<BPA_RecipeAttributeValue>();
        //        var baseConfigDelList = new List<BPA_RecipeBaseConfig>();
        //        var baseDetailDelList = new List<BPA_RecipeConfigDetail>();
        //        var attrConfigDelList = new List<BPA_RecipeConfig>();
        //        var attrDetailDelList = new List<BPA_RecipeConfigDetail>();
        //        foreach (var item in dto)
        //        {
        //            var recipe = recipeList.FirstOrDefault(t => t.Id == item.Id);
        //            if (recipe == null)
        //            {
        //                errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方删除失败：未找到配方数据", Id = item.Id, Name = item.Name });
        //                continue;
        //            }
        //            var ishave = relationList.Any(t => t.RecipeId == item.Id);
        //            if (ishave)
        //            {
        //                errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "配方删除失败：该配方已有门店和设备使用", Id = item.Id, Name = item.Name });
        //                continue;
        //            }
        //            recipe.Name = "";
        //            recipe.ReferenceWeight = null;
        //            recipeUpList.Add(recipe);
        //            var attrDataList = attrList.Where(t => t.Id == item.Id);
        //            attrDelList.AddRange(attrDataList);
        //            valueDelList.AddRange(valueList.Where(t => attrDataList.Select(a => a.Id).Contains(t.AttributeId)));
        //            var baseConfigDataList = baseConfigList.Where(t => t.RecipeId == item.Id);
        //            baseConfigDelList.AddRange(baseConfigDataList);
        //            baseDetailDelList.AddRange(detailList.Where(t => baseConfigDataList.Select(a => a.Id).Contains(t.ConfigId)));
        //            var attrConfigDataList = attrConfigList.Where(t => t.RecipeId == item.Id);
        //            attrConfigDelList.AddRange(attrConfigDataList);
        //            attrDetailDelList.AddRange(detailList.Where(t => attrConfigDataList.Select(a => a.Id).Contains(t.ConfigId)));
        //            successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方删除成功", Id = item.Id, Name = item.Name });
        //        }
        //        if (errorList.Count > 0)
        //            return errorList;
        //        await _db.Updateable(recipeUpList).ExecuteCommandAsync();
        //        await _db.Deleteable(attrDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(valueDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(baseConfigDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(baseDetailDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(attrConfigDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(attrDetailDelList).ExecuteCommandAsync();
        //        _db.Ado.CommitTran();
        //        return successList;
        //    }
        //    catch (Exception e)
        //    {
        //        _db.Ado.RollbackTran();
        //        throw Oops.Oh(e.Message);
        //    }
        //}
        #endregion

        #region 配方版本
        /// <summary>
        /// 上传更新配方版本
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeVersion(List<RecipeVersionDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var recipeList = await _db.Queryable<BPA_RecipeInfo>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var versionAddList = new List<BPA_RecipeVersion>();
                var versionUpList = new List<BPA_RecipeVersion>();
                foreach (var item in dto)
                {
                    var recipe = recipeList.FirstOrDefault(t => t.Id == item.RecipeId);
                    if (recipe == null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方版本上传失败：未找到配方信息", Id = item.RecipeId, Name = item.RecipeName });
                        continue;
                    }
                    foreach (var version in item.VersionList)
                    {
                        var repeatData = versionList.FirstOrDefault(t => t.RecipeId == item.RecipeId && t.Version == version.Version && t.Id != version.Id);
                        if (repeatData != null)
                        {
                            errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "配方版本上传失败：重复配方版本数据", Id = version.Id, Name = version.Version.ToString() });
                            continue;
                        }
                        var versionData = versionList.FirstOrDefault(t => t.Id == version.Id);
                        if (versionData == null)
                        {
                            versionAddList.Add(new BPA_RecipeVersion
                            {
                                Id = version.Id,
                                RecipeId = item.RecipeId,
                                Version = version.Version,
                                Describe = version.Describe,
                                AutoKey = autoKey
                            });
                        }
                        else
                        {
                            versionData.Version = version.Version;
                            versionData.Describe = version.Describe;
                            versionUpList.Add(versionData);
                        }
                        successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方版本上传成功", Id = version.Id, Name = version.Version.ToString() });
                    }
                }
                if (errorList.Count > 0)
                    return errorList;
                if (versionAddList.Count > 0)
                    await _db.Insertable(versionAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                await _db.Updateable(versionUpList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 配方多属性
        ///// <summary>
        ///// 获取配方多属性
        ///// </summary>
        ///// <param name="recipeId"></param>
        ///// <returns></returns>
        //public async Task<List<RecipeAttrDto>> GetRecipeAttr(string recipeId)
        //{
        //    var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
        //    return await _db.Queryable<BPA_RecipeAttribute>().Where(t => t.RecipeId == recipeId)
        //        .OrderBy(t => t.Sort)
        //        .Select(t => new RecipeAttrDto
        //        {
        //            Id = t.Id,
        //            RecipeId = t.RecipeId,
        //            Name = t.Name,
        //            Sort = t.Sort,
        //        }).Mapper(a => a.ValueList = valueList.Where(t => t.AttributeId == a.Id).OrderBy(m => m.Sort).Select(b => new RecipeAttrValueDto
        //        {
        //            Id = b.Id,
        //            AttributeId = b.AttributeId,
        //            Name = b.Name,
        //            Sort = b.Sort,
        //        }).ToList()).ToListAsync();
        //}

        /// <summary>
        /// 上传更新配方多属性
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeAttr(List<RecipeAttrDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var attrList = await _db.Queryable<BPA_RecipeAttribute>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var attrAddList = new List<BPA_RecipeAttribute>();
                var valueAddList = new List<BPA_RecipeAttributeValue>();
                var attrUpList = new List<BPA_RecipeAttribute>();
                var valueUpList = new List<BPA_RecipeAttributeValue>();
                foreach (var item in dto)
                {
                    var version = versionList.FirstOrDefault(t => t.Id == item.RecipeVersionId);
                    if (version == null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方多属性上传失败：未找到配方分组信息", Id = item.RecipeVersionId, Name = item.RecipeVersionName });
                        continue;
                    }
                    foreach (var attr in item.AttrList)
                    {
                        var attrData = attrList.FirstOrDefault(t => t.Id == attr.Id);
                        if (attrData == null)
                        {
                            attrAddList.Add(new BPA_RecipeAttribute
                            {
                                Id = attr.Id,
                                RecipeVersionId = item.RecipeVersionId,
                                Name = attr.Name,
                                Sort = attr.Sort,
                                AutoKey = autoKey
                            });
                        }
                        else
                        {
                            attrData.Name = attr.Name;
                            attrData.Sort = attr.Sort;
                            attrUpList.Add(attrData);
                        }
                        foreach (var value in attr.ValueList)
                        {
                            var valueData = valueList.FirstOrDefault(t => t.Id == value.Id);
                            if (valueData == null)
                            {
                                valueAddList.Add(new BPA_RecipeAttributeValue
                                {
                                    Id = value.Id,
                                    AttributeId = attr.Id,
                                    Name = value.Name,
                                    Sort = value.Sort,
                                    AutoKey = autoKey
                                });
                            }
                            else
                            {
                                valueData.Name = value.Name;
                                valueData.Sort = value.Sort;
                                valueUpList.Add(valueData);
                            }
                            successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方属性值上传成功", Id = value.Id, Name = value.Name });
                        }
                        successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方属性上传成功", Id = attr.Id, Name = attr.Name });
                    }
                }
                if (errorList.Count > 0)
                    return errorList;
                if (attrAddList.Count > 0)
                    await _db.Insertable(attrAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (valueAddList.Count > 0)
                    await _db.Insertable(valueAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                await _db.Updateable(attrUpList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                await _db.Updateable(valueUpList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        ///// <summary>
        ///// 删除配方多属性
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeAttr(List<AttrValueDelDto> dto)
        //{
        //    try
        //    {
        //        _db.Ado.BeginTran();
        //        var errorList = new List<InterfaceMessageDto>();
        //        var successList = new List<InterfaceMessageDto>();
        //        var recipeList = await _db.Queryable<BPA_RecipeInfo>().ToListAsync();
        //        var attrList = await _db.Queryable<BPA_RecipeAttribute>().ToListAsync();
        //        var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
        //        var configList = await _db.Queryable<BPA_RecipeConfig>().ToListAsync();
        //        var attrDelList = new List<BPA_RecipeAttribute>();
        //        var valueDelList = new List<BPA_RecipeAttributeValue>();
        //        foreach (var item in dto)
        //        {
        //            var recipe = recipeList.FirstOrDefault(t => t.Id == item.RecipeId);
        //            if (recipe == null)
        //            {
        //                errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方多属性删除失败：未找到配方数据", Id = item.RecipeId, Name = item.RecipeName });
        //                continue;
        //            }
        //            var attrDataList = attrList.Where(t => t.RecipeId == item.RecipeId);
        //            var valueDataList = valueList.Where(t => attrDataList.Select(a => a.Id).Contains(t.AttributeId));
        //            if (string.IsNullOrWhiteSpace(item.AttrId))
        //            {
        //                foreach (var valueData in valueDataList)
        //                {
        //                    if (configList.Any(t => t.AttributeId.Contains(valueData.Id)))
        //                    {
        //                        errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "配方多属性删除失败：该配方的多属性已被使用", Id = item.RecipeId, Name = item.RecipeName });
        //                        continue;
        //                    }
        //                }
        //                attrDelList.AddRange(attrDataList);
        //                valueDelList.AddRange(valueDataList);
        //                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方多属性删除成功", Id = item.RecipeId, Name = item.RecipeName });
        //            }
        //            else
        //            {
        //                var attrDataVList = attrDataList.Where(t => t.Id == item.AttrId);
        //                var valueDataVList = valueDataList.Where(t => attrDataVList.Select(a => a.Id).Contains(t.AttributeId));
        //                if (string.IsNullOrWhiteSpace(item.ValueId))
        //                {
        //                    foreach (var attrDataV in attrDataVList)
        //                    {
        //                        if (configList.Any(t => t.AttributeId.Contains(attrDataV.Id)))
        //                        {
        //                            errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "配方多属性删除失败：该属性已被使用", Id = item.AttrId, Name = item.AttrName });
        //                            continue;
        //                        }
        //                    }
        //                    attrDelList.AddRange(attrDataVList);
        //                    valueDelList.AddRange(valueDataVList);
        //                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方属性删除成功", Id = item.AttrId, Name = item.AttrName });
        //                }
        //                else
        //                {
        //                    if (configList.Any(t => t.AttributeId.Contains(item.ValueId)))
        //                    {
        //                        errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "配方多属性删除失败：该属性值已被使用", Id = item.ValueId, Name = item.ValueName });
        //                        continue;
        //                    }
        //                    valueDelList.AddRange(valueDataVList.Where(t => t.Id == item.ValueId));
        //                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方属性值删除成功", Id = item.ValueId, Name = item.ValueName });
        //                }
        //            }
        //        }
        //        if (errorList.Count > 0)
        //            return errorList;
        //        await _db.Deleteable(attrDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(valueDelList).ExecuteCommandAsync();
        //        _db.Ado.CommitTran();
        //        return successList;
        //    }
        //    catch (Exception e)
        //    {
        //        _db.Ado.RollbackTran();
        //        throw Oops.Oh(e.Message);
        //    }
        //}
        #endregion

        #region 配方物料
        ///// <summary>
        ///// 获取配方物料
        ///// </summary>
        ///// <param name="recipeId"></param>
        ///// <returns></returns>
        //public async Task<List<RecipeBatchingDto>> GetRecipeBatching(string recipeId)
        //{
        //    var result = new List<RecipeBatchingDto>();
        //    var nullBatchingList = await _db.Queryable<BPA_Batching>().OrderBy(t => t.Name).Select(a => new RecipeBatchingListDto
        //    {
        //        BatchingId = a.Id,
        //        BatchingName = a.Name,
        //        Weight = 0
        //    }).ToListAsync();
        //    var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
        //    var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
        //    var baseConfig = await _db.Queryable<BPA_RecipeBaseConfig>().FirstAsync(t => t.RecipeId == recipeId);
        //    if (baseConfig == null)
        //    {
        //        result.Add(new RecipeBatchingDto
        //        {
        //            AttributeId = "",
        //            AttributeName = "",
        //            BatchingList = nullBatchingList
        //        });
        //    }
        //    else
        //    {
        //        var baseDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => t.ConfigId == baseConfig.Id).ToListAsync();
        //        result.Add(new RecipeBatchingDto
        //        {
        //            AttributeId = "",
        //            AttributeName = "基础配方",
        //            BatchingList = (from batching in batchingList
        //                            select new RecipeBatchingListDto
        //                            {
        //                                BatchingId = batching.Id,
        //                                BatchingName = batching.Name,
        //                                Weight = baseDetailList.FirstOrDefault(t => t.BatchingId == batching.Id)?.Weight ?? 0
        //                            }).OrderByDescending(a => a.Weight).ToList()
        //        });
        //    }
        //    var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.RecipeId == recipeId).ToListAsync();
        //    var attrDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => attrConfigList.Select(a => a.Id).Contains(t.ConfigId)).ToListAsync();
        //    foreach (var attr in attrConfigList)
        //    {
        //        var attrConfigDetailList = attrDetailList.Where(t => t.ConfigId == attr.Id).ToList();
        //        result.Add(new RecipeBatchingDto
        //        {
        //            AttributeId = attr.AttributeId,
        //            AttributeName = GetAttributeName(attr.AttributeId, valueList),
        //            BatchingList = !attrConfigDetailList.Any() ? nullBatchingList : (from batching in batchingList
        //                                                                             select new RecipeBatchingListDto
        //                                                                             {
        //                                                                                 BatchingId = batching.Id,
        //                                                                                 BatchingName = batching.Name,
        //                                                                                 Weight = attrConfigDetailList.FirstOrDefault(t => t.BatchingId == batching.Id)?.Weight ?? 0
        //                                                                             }).OrderByDescending(a => a.Weight).ToList()
        //        });
        //    }
        //    return result;
        //}

        /// <summary>
        /// 上传更新配方物料
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeBatching(List<RecipeBatchingDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var versionList = await _db.Queryable<BPA_RecipeVersion>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var configDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var batchingList = await _db.Queryable<BPA_Batching>().ToListAsync();
                var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var baseConfigAddList = new List<BPA_RecipeBaseConfig>();
                var attrConfigAddList = new List<BPA_RecipeConfig>();
                var configDetailAddList = new List<BPA_RecipeConfigDetail>();
                var configDetailDelList = new List<BPA_RecipeConfigDetail>();
                foreach (var item in dto)
                {
                    var version = versionList.FirstOrDefault(t => t.Id == item.RecipeVersionId);
                    if (version == null)
                    {
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料上传失败：未找到配方信息", Id = item.RecipeVersionId, Name = item.RecipeVersionName });
                        continue;
                    }
                    foreach (var config in item.ConfigList)
                    {
                        if (string.IsNullOrWhiteSpace(config.AttributeId))
                        {
                            var repeatData = baseConfigList.FirstOrDefault(t => t.Id == config.Id && t.RecipeVersionId != item.RecipeVersionId);
                            if (repeatData != null)
                            {
                                errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "配方物料上传失败：重复基础配方物料数据", Id = config.Id, Name = config.Id });
                                continue;
                            }
                            var baseConfig = baseConfigList.FirstOrDefault(t => t.Id == config.Id);
                            if (baseConfig == null)
                            {
                                baseConfigAddList.Add(new BPA_RecipeBaseConfig
                                {
                                    Id = config.Id,
                                    RecipeVersionId = item.RecipeVersionId,
                                    AutoKey = autoKey
                                });
                            }
                            configDetailDelList.AddRange(configDetailList.Where(t => t.ConfigId == config.Id));
                            foreach (var batching in config.BatchingList)
                            {
                                if (batchingList.FirstOrDefault(t => t.Id == batching.BatchingId) == null)
                                {
                                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料上传失败：未找到物料信息", Id = batching.BatchingId, Name = batching.BatchingName });
                                    continue;
                                }
                                configDetailAddList.Add(new BPA_RecipeConfigDetail
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    ConfigId = config.Id,
                                    BatchingId = batching.BatchingId,
                                    Weight = batching.Weight,
                                    AutoKey = autoKey
                                });
                                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方物料上传成功", Id = batching.BatchingId, Name = batching.BatchingName });
                            }
                        }
                        else
                        {
                            var repeatData = attrConfigList.FirstOrDefault(t => t.Id == config.Id && !CompareAttrId(t.AttributeId, config.AttributeId));
                            if (repeatData != null)
                            {
                                errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "配方物料上传失败：重复多属性配方物料数据", Id = config.AttributeId, Name = config.AttributeName });
                                continue;
                            }
                            foreach (var attrId in config.AttributeId.Split(','))
                            {
                                if (!valueList.Any(t => t.Id == attrId))
                                {
                                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料上传失败：未找到多属性信息", Id = config.AttributeId, Name = config.AttributeName });
                                    continue;
                                }
                            }
                            var attrConfig = attrConfigList.FirstOrDefault(t => t.Id == config.Id);
                            if (attrConfig == null)
                            {
                                attrConfigAddList.Add(new BPA_RecipeConfig
                                {
                                    Id = config.Id,
                                    RecipeVersionId = item.RecipeVersionId,
                                    AttributeId = config.AttributeId,
                                    AutoKey = autoKey
                                });
                            }
                            configDetailDelList.AddRange(configDetailList.Where(t => t.ConfigId == config.Id));
                            foreach (var batching in config.BatchingList)
                            {
                                if (batchingList.FirstOrDefault(t => t.Id == batching.BatchingId) == null)
                                {
                                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料上传失败：未找到物料信息", Id = batching.BatchingId, Name = batching.BatchingName });
                                    continue;
                                }
                                configDetailAddList.Add(new BPA_RecipeConfigDetail
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    ConfigId = config.Id,
                                    BatchingId = batching.BatchingId,
                                    Weight = batching.Weight,
                                    AutoKey = autoKey
                                });
                                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方物料上传成功", Id = batching.BatchingId, Name = batching.BatchingName });
                            }
                        }
                    }
                }
                if (errorList.Count > 0)
                    return errorList;
                await _db.Deleteable(configDetailDelList).ExecuteCommandAsync();
                if (baseConfigAddList.Count > 0)
                    await _db.Insertable(baseConfigAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (attrConfigAddList.Count > 0)
                    await _db.Insertable(attrConfigAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (configDetailAddList.Count > 0)
                    await _db.Insertable(configDetailAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        ///// <summary>
        ///// 删除配方物料
        ///// </summary>
        ///// <param name="dto"></param>
        ///// <returns></returns>
        //public async Task<List<InterfaceMessageDto>> DeleteRecipeBatching(List<RecipeBatchingDelDto> dto)
        //{
        //    try
        //    {
        //        _db.Ado.BeginTran();
        //        var errorList = new List<InterfaceMessageDto>();
        //        var successList = new List<InterfaceMessageDto>();
        //        var recipeList = await _db.Queryable<BPA_RecipeInfo>().ToListAsync();
        //        var baseConfigList = await _db.Queryable<BPA_RecipeBaseConfig>().ToListAsync();
        //        var attrConfigList = await _db.Queryable<BPA_RecipeConfig>().ToListAsync();
        //        var configDetailList = await _db.Queryable<BPA_RecipeConfigDetail>().ToListAsync();
        //        var baseConfigDelList = new List<BPA_RecipeBaseConfig>();
        //        var attrConfigDelList = new List<BPA_RecipeConfig>();
        //        var configDetailDelList = new List<BPA_RecipeConfigDetail>();
        //        foreach (var item in dto)
        //        {
        //            var recipe = recipeList.FirstOrDefault(t => t.Id == item.RecipeId);
        //            if (recipe == null)
        //            {
        //                errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料删除失败：未找到配方数据", Id = item.RecipeId, Name = item.RecipeName });
        //                continue;
        //            }
        //            var baseConfigData = baseConfigList.FirstOrDefault(t => t.RecipeId == item.RecipeId);
        //            var baseDetailDataList = configDetailList.Where(t => t.ConfigId == baseConfigData?.Id);
        //            var attrConfigDataList = attrConfigList.Where(t => t.RecipeId == item.RecipeId);
        //            var attrDetailDataList = configDetailList.Where(t => attrConfigDataList.Select(a => a.Id).Contains(t.ConfigId));
        //            if (string.IsNullOrWhiteSpace(item.ConfigId))
        //            {
        //                baseConfigDelList.Add(baseConfigData);
        //                configDetailDelList.AddRange(baseDetailDataList);
        //                attrConfigDelList.AddRange(attrConfigDataList);
        //                configDetailDelList.AddRange(attrDetailDataList);
        //                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方物料删除成功", Id = item.RecipeId, Name = item.RecipeName });
        //            }
        //            else
        //            {
        //                if (baseConfigData?.Id == item.ConfigId)
        //                {
        //                    baseConfigDelList.Add(baseConfigData);
        //                    configDetailDelList.AddRange(baseDetailDataList);
        //                }
        //                else
        //                {
        //                    var attrConfig = attrConfigDataList.FirstOrDefault(t => t.Id == item.ConfigId);
        //                    if (attrConfig == null)
        //                    {
        //                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "配方物料删除失败：未找到配置数据", Id = item.ConfigId, Name = item.ConfigName });
        //                        continue;
        //                    }
        //                    else
        //                    {
        //                        attrConfigDelList.Add(attrConfig);
        //                        configDetailDelList.AddRange(attrDetailDataList.Where(t => t.ConfigId == item.ConfigId));
        //                    }
        //                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方物料删除成功", Id = item.ConfigId, Name = item.ConfigName });
        //                }
        //            }
        //        }
        //        if (errorList.Count > 0)
        //            return errorList;
        //        await _db.Deleteable(baseConfigDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(attrConfigDelList).ExecuteCommandAsync();
        //        await _db.Deleteable(configDetailDelList).ExecuteCommandAsync();
        //        _db.Ado.CommitTran();
        //        return successList;
        //    }
        //    catch (Exception e)
        //    {
        //        _db.Ado.RollbackTran();
        //        throw Oops.Oh(e.Message);
        //    }
        //}

        public static string GetAttributeName(string valueIds, List<BPA_RecipeAttributeValue> valueList)
        {
            var valueIdList = valueIds.Split(',');
            var names = "";
            foreach (var valueId in valueIdList)
            {
                var name = valueList.FirstOrDefault(t => t.Id == valueId)?.Name;
                names = names + "-" + name;
            }
            return names.TrimStart('-');
        }

        public static bool CompareAttrId(string attributeIds, string attrIds)
        {
            var attributeIdSet = new HashSet<string>(attributeIds.Split(',').ToList());
            var attrIdSet = new HashSet<string>(attrIds.Split(',').ToList());
            return attributeIdSet.SetEquals(attrIdSet);
        }
        #endregion

        #region 配方模版
        /// <summary>
        /// 上传更新配方模版
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeTemplate(List<RecipeTemplateDataDto> dto)
        {
            try
            {
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                if (!dto.Any())
                {
                    errorList.Add(new InterfaceMessageDto { Code = 30001, Message = "配方模版上传失败：上传数据为空", Id = "", Name = "" });
                    return errorList;
                }
                var groupList = await _db.Queryable<BPA_RecipeGroup>().Where(t => t.AutoKey == autoKey).ToListAsync();
                var templateList = await _db.Queryable<BPA_RecipeTemplate>().Where(t => t.AutoKey == autoKey).ToListAsync();
                if(groupList.Any())
                {
                    var ids = string.Join(",", groupList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                    string sql = $"DELETE FROM BPA_RecipeGroup WHERE (Id, AutoKey) IN ({ids});";
                    if(templateList.Any())
                    {
                        var tIds = string.Join(",", templateList.Select(r => $"('{r.Id}', {r.AutoKey})"));
                        sql += $"DELETE FROM BPA_RecipeTemplate WHERE (Id, AutoKey) IN ({tIds});";
                    }
                    await _db.Ado.ExecuteCommandAsync(sql);
                }
                _db.Ado.BeginTran();
                var groupAddList = new List<BPA_RecipeGroup>();
                var templateAddList = new List<BPA_RecipeTemplate>();
                foreach (var item in dto)
                {
                    groupAddList.Add(new BPA_RecipeGroup
                    {
                        Id = item.Id,
                        Name = item.Name,
                        IsWeight = item.IsWeight,
                        Sort = item.Sort,
                        AutoKey = autoKey
                    });
                    foreach (var template in item.TemplateList)
                    {
                        templateAddList.Add(new BPA_RecipeTemplate
                        {
                            Id = template.Id,
                            RecipeGroupId = item.Id,
                            RecipeId = template.RecipeId,
                            VersionId = template.VersionId,
                            Sort = template.Sort,
                            AutoKey = autoKey
                        });
                        successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方模版上传成功", Id = template.Id, Name = template.Id });
                    }
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方分组上传成功", Id = item.Id, Name = item.Name });
                }
                if (errorList.Count > 0)
                    return errorList;
                if (groupAddList.Count > 0)
                    await _db.Insertable(groupAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (templateAddList.Count > 0)
                    await _db.Insertable(templateAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #endregion

        #region 报表接口
        /// <summary>
        /// 上传配方使用数据
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadRecipeUseDetail(List<RecipeUseDetailDataDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var detailList = await _db.Queryable<BPA_RecipeUseDetail>().ToListAsync();
                var recipeUseAddList = new List<BPA_RecipeUseDetail>();
                var recipeBatchingUseAddList = new List<BPA_RecipeBathingUseDetail>();
                var autoKey = GetAutoKeyFromHeaders();
                foreach (var item in dto)
                {
                    if (detailList.Any(t => t.RecipeName == item.RecipeName && t.AttributeName == item.AttributeName && t.StoreName == item.StoreName && t.DeviceName == item.DeviceName && t.UseDate == item.UseDate))
                        continue;
                    var id = Guid.NewGuid().ToString();
                    recipeUseAddList.Add(new BPA_RecipeUseDetail
                    {
                        Id = id,
                        IsWeight = item.IsWeight,
                        Status = item.Status,
                        RecipeName = item.RecipeName,
                        AttributeName = item.AttributeName,
                        StoreName = item.StoreName,
                        DeviceName = item.DeviceName,
                        JsonData = JsonConvert.SerializeObject(item),
                        UseDate = item.UseDate
                    });
                    foreach (var batching in item.BatchingList)
                    {
                        if (batching.UseWeight != 0)
                        {
                            recipeBatchingUseAddList.Add(new BPA_RecipeBathingUseDetail
                            {
                                RecipeUseId = id,
                                BatchingName = batching.BatchingName,
                                UseWeight = batching.UseWeight,
                                StoreName = item.StoreName,
                                DeviceName = item.DeviceName,
                                UseDate = item.UseDate,
                            });
                        }
                    }
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "配方使用数据上传成功", Id = id, Name = item.RecipeName + "(" + item.AttributeName + ")" });
                }
                if (errorList.Count > 0)
                    return errorList;
                if (recipeUseAddList.Count > 0)
                    await _db.Insertable(recipeUseAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (recipeBatchingUseAddList.Count > 0)
                    await _db.Insertable(recipeBatchingUseAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 日志接口
        /// <summary>
        /// 上传设备日志
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadDeviceLog(DeviceLogDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var logAddList = new List<BPA_DeviceLog>();
                var aliyunHost = "https://bpa.oss-cn-chengdu.aliyuncs.com/";
                var data = await _aliyunOssClient.UpFileAsync(dto.FormFile);
                var log = new BPA_DeviceLog
                {
                    Id = Guid.NewGuid().ToString(),
                    FileName = dto.FormFile.FileName,
                    FileUrl = aliyunHost + data,
                    StoreName = dto.StoreName,
                    DeviceName = dto.DeviceName
                };
                logAddList.Add(log);
                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "设备日志上传成功", Id = log.Id, Name = log.FileName });
                if (errorList.Count > 0)
                    return errorList;
                if (logAddList.Count > 0)
                    await _db.Insertable(logAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion

        #region 激活接口
        /// <summary>
        /// 在线激活码激活设备
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> ActivationDevice(ActivationDeviceDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                if (dto.OnlineKey.Length == 14)
                {
                    var activation = await _db.Queryable<BPA_ActivationInfo>().ClearFilter().FirstAsync(t => t.OnlineKey == dto.OnlineKey);
                    if (activation == null)
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "在线激活失败：未找到在线激活码数据", Id = dto.OnlineKey, Name = dto.OnlineKey });
                    if (activation.Status == 2)
                        errorList.Add(new InterfaceMessageDto { Code = 30003, Message = "在线激活失败：在线激活码已被使用", Id = dto.OnlineKey, Name = dto.OnlineKey });
                    activation.Status = 2;
                    activation.AutoKey = autoKey;
                    activation.GroupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value;
                    await _db.Updateable(activation).ExecuteCommandAsync();
                    var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.AutoKey == autoKey);
                    if (device == null)
                        errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "在线激活失败：未找到设备", Id = autoKey.ToString(), Name = autoKey.ToString() });
                    device.IsActivation = 1;
                    await _db.Updateable(device).ExecuteCommandAsync();
                    var onlineLog = new BPA_ActivationLog
                    {
                        ActivationId = activation.Id,
                        OnlineKey = dto.OnlineKey,
                        ActivationDate = DateTime.Now,
                        AutoKey = autoKey
                    };
                    await _db.Insertable(onlineLog).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    successList.Add(new InterfaceMessageDto { Code = 30000, Message = "在线激活成功", Id = dto.OnlineKey, Name = device.Name });
                }
                else
                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "在线激活失败：未使用在线激活码", Id = dto.OnlineKey, Name = dto.OnlineKey });
                if (errorList.Count > 0)
                    return errorList;
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 上传离线激活记录
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<InterfaceMessageDto>> UploadOfflineLog(OfflineActivationLogDto dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var autoKey = GetAutoKeyFromHeaders();
                var errorList = new List<InterfaceMessageDto>();
                var successList = new List<InterfaceMessageDto>();
                var activation = await _db.Queryable<BPA_ActivationInfo>().ClearFilter().FirstAsync(t => t.OfflineKey == dto.OfflineKey);
                if (activation == null)
                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "激活记录上传失败：未找到离线激活码数据", Id = dto.OfflineKey, Name = dto.OfflineKey });
                activation.Status = 2;
                await _db.Updateable(activation).ExecuteCommandAsync();
                var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.AutoKey == autoKey);
                if (device == null)
                    errorList.Add(new InterfaceMessageDto { Code = 30002, Message = "激活记录上传失败：未找到设备数据", Id = dto.OfflineKey, Name = dto.OfflineKey });
                device.IsActivation = 1;
                await _db.Updateable(device).ExecuteCommandAsync();
                var offlineLog = new BPA_ActivationLog
                {
                    ActivationId = activation.Id,
                    OfflineKey = dto.OfflineKey,
                    ActivationDate = dto.ActivationDate,
                    AutoKey = autoKey
                };
                await _db.Insertable(offlineLog).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                successList.Add(new InterfaceMessageDto { Code = 30000, Message = "激活记录上传成功", Id = dto.OfflineKey, Name = dto.OfflineKey });
                if (errorList.Count > 0)
                    return errorList;
                _db.Ado.CommitTran();
                return successList;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
        #endregion
    }
}
