﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.RecipeAttribute;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.Recipe;
using BPA.SAAS.Manage.Core.RecipeAttribute;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class RecipeAttributeService : IRecipeAttributeService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public RecipeAttributeService(ISqlSugarClient db)
        {
            _db = db;
        }

        /// <summary>
        /// 通过配方Id获取配方属性
        /// </summary>
        /// <param name="recipeVersionId"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<List<RecipeAttributeDto>> GetRecipeAttributeList(string recipeVersionId, int autoKey)
        {
            var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
            return await _db.Queryable<BPA_RecipeAttribute>().Where(t => t.RecipeVersionId == recipeVersionId && t.AutoKey == autoKey)
                .OrderBy(t => t.Sort)
                .Select(t => new RecipeAttributeDto
                {
                    Id = t.Id,
                    RecipeVersionId = t.RecipeVersionId,
                    Name = t.Name,
                    Sort = t.Sort,
                }).Mapper(a => a.ValueList = valueList.Where(t => t.AttributeId == a.Id && t.AutoKey == autoKey).OrderBy(m => m.Sort).Select(b => new RecipeAttributeValueDto
                {
                    Id = b.Id,
                    AttributeId = b.AttributeId,
                    Name = b.Name,
                    Sort = b.Sort,
                }).ToList()).ToListAsync();
        }

        /// <summary>
        /// 更新配方属性
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateRecipeAttribute(List<RecipeAttributeDto> dto)
        {
            try
            {
                _db.Ado.BeginTran();
                var attrList = await _db.Queryable<BPA_RecipeAttribute>().ToListAsync();
                var attrValueList = await _db.Queryable<BPA_RecipeAttributeValue>().ToListAsync();
                var attrAddList = new List<BPA_RecipeAttribute>();
                var attrValueAddList = new List<BPA_RecipeAttributeValue>();
                var attrUpdateList = new List<BPA_RecipeAttribute>();
                var attrValueUpdateList = new List<BPA_RecipeAttributeValue>();
                foreach (var item in dto)
                {
                    if (item.Id == null)
                    {
                        var id = Guid.NewGuid().ToString();
                        attrAddList.Add(new BPA_RecipeAttribute
                        {
                            Id = id,
                            Name = item.Name,
                            RecipeVersionId = item.RecipeVersionId,
                            Sort = item.Sort,
                            AutoKey = item.AutoKey
                        });
                        if (item.ValueList != null)
                        {
                            foreach (var value in item.ValueList)
                            {
                                attrValueAddList.Add(new BPA_RecipeAttributeValue
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    AttributeId = id,
                                    Name = value.Name,
                                    Sort = value.Sort,
                                    AutoKey = item.AutoKey
                                });
                            }
                        }
                    }
                    else
                    {
                        var id = string.IsNullOrWhiteSpace(item.Id) ? Guid.NewGuid().ToString() : item.Id;
                        var attr = attrList.FirstOrDefault(t => t.Id == item.Id && t.AutoKey == item.AutoKey);
                        if (attr == null)
                        {
                            attrAddList.Add(new BPA_RecipeAttribute
                            {
                                Id = id,
                                Name = item.Name,
                                RecipeVersionId = item.RecipeVersionId,
                                Sort = item.Sort,
                                AutoKey = item.AutoKey
                            });
                        }
                        else
                        {
                            attr.Name = item.Name;
                            attr.Sort = item.Sort;
                            attrUpdateList.Add(attr);
                        }
                        if (item.ValueList != null)
                        {
                            foreach (var value in item.ValueList)
                            {
                                var attrValue = attrValueList.FirstOrDefault(t => t.Id == value.Id && t.AutoKey == item.AutoKey);
                                if (attrValue == null)
                                {
                                    attrValueAddList.Add(new BPA_RecipeAttributeValue
                                    {
                                        Id = Guid.NewGuid().ToString(),
                                        AttributeId = id,
                                        Name = value.Name,
                                        Sort = value.Sort,
                                        AutoKey = item.AutoKey
                                    });
                                }
                                else
                                {
                                    attrValue.Name = value.Name;
                                    attrValue.Sort = value.Sort;
                                    attrValueUpdateList.Add(attrValue);
                                }
                            }
                        }
                    }
                }
                if (attrAddList.Any())
                    await _db.Insertable(attrAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                if (attrValueAddList.Any())
                    await _db.Insertable(attrValueAddList).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                await _db.Updateable(attrUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                await _db.Updateable(attrValueUpdateList).IgnoreColumns(t => t.AutoKey).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除配方属性/属性值
        /// </summary>
        /// <param name="id"></param>
        /// <param name="type"></param>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<bool> DeleteRecipeAttribute(string id, int type, int autoKey)
        {
            try
            {
                _db.Ado.BeginTran();
                var configList = await _db.Queryable<BPA_RecipeConfig>().ToListAsync();
                if (type == 1)
                {
                    var attr = await _db.Queryable<BPA_RecipeAttribute>().FirstAsync(t => t.Id == id && t.AutoKey == autoKey);
                    var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => t.AttributeId == id && t.AutoKey == autoKey).ToListAsync();
                    foreach (var value in valueList)
                    {
                        var isUse = configList.Any(t => t.AttributeId.Contains(value.Id) && t.AutoKey == autoKey);
                        if (isUse)
                            return false;
                    }
                    await _db.Deleteable<BPA_RecipeAttribute>().Where(t => t.Id == id && t.AutoKey == autoKey).ExecuteCommandAsync();
                    if (valueList.Any())
                    {
                        var ids = string.Join(",", valueList.Select(t => $"('{t.Id}', {t.AutoKey})"));
                        if (!string.IsNullOrWhiteSpace(ids))
                        {
                            string sql = $"DELETE FROM BPA_RecipeAttributeValue WHERE (Id, AutoKey) IN ({ids})";
                            await _db.Ado.ExecuteCommandAsync(sql);
                        }
                    }
                }
                else
                {
                    var isUse = configList.Any(t => t.AttributeId.Contains(id) && t.AutoKey == autoKey);
                    if (isUse)
                        return false;
                    var valueList = await _db.Queryable<BPA_RecipeAttributeValue>().Where(t => t.Id == id).ToListAsync();
                    if (valueList.Any())
                    {
                        var ids = string.Join(",", valueList.Select(t => $"('{t.Id}', {t.AutoKey})"));
                        if (!string.IsNullOrWhiteSpace(ids))
                        {
                            string sql = $"DELETE FROM BPA_RecipeAttributeValue WHERE (Id, AutoKey) IN ({ids})";
                            await _db.Ado.ExecuteCommandAsync(sql);
                        }
                    }
                }
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
    }
}
