﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Report;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Recipe;
using BPA.SAAS.Manage.Core.RecipeAttribute;
using BPA.SAAS.Manage.Core.Store;
using Newtonsoft.Json;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class ReportService : IReportService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public ReportService(ISqlSugarClient db)
        {
            _db = db;
        }

        /// <summary>
        /// 获取配方使用明细
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<RecipeUseDetailDto>> GetRecipeUseDetail(RecipeQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrWhiteSpace(dto.RecipeName))
                conModels.Add(new ConditionalModel() { FieldName = "RecipeName", ConditionalType = ConditionalType.Like, FieldValue = dto.RecipeName });
            if (!string.IsNullOrWhiteSpace(dto.StoreName))
                conModels.Add(new ConditionalModel() { FieldName = "StoreName", ConditionalType = ConditionalType.Like, FieldValue = dto.StoreName });
            if (!string.IsNullOrWhiteSpace(dto.DeviceName))
                conModels.Add(new ConditionalModel() { FieldName = "DeviceName", ConditionalType = ConditionalType.Like, FieldValue = dto.DeviceName });
            if (dto.StartTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.GreaterThanOrEqual, FieldValue = dto.StartTime.ToString() });
            if (dto.EndTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.LessThanOrEqual, FieldValue = dto.EndTime.ToString() });
            var res = await _db.Queryable<BPA_RecipeUseDetail>().Where(conModels)
                .WhereIF(dto.IsWeight != null, x => x.IsWeight == dto.IsWeight)
                .WhereIF(dto.Status != null, x => x.Status == dto.Status)
                .OrderByDescending(a => a.UseDate)
                .Select(a => new RecipeUseDetailDto
                {
                    Id = a.Id,
                    RecipeName = $"{a.RecipeName}({a.AttributeName})",
                    StoreName = a.StoreName,
                    DeviceName = a.DeviceName,
                    IsWeight = a.IsWeight,
                    Status = a.Status,
                    UseDate = a.UseDate,
                }).ToListAsync();
            return res;
        }

        public async Task<string> DownRecipeUseDetail(RecipeQueryDto dto)
        {
            var data = await GetRecipeUseDetail(dto);
            IWorkbook workbook = new XSSFWorkbook();
            ISheet sheet = workbook.CreateSheet("配方使用明细");
            sheet.SetColumnWidth(0, 20 * 256);
            sheet.SetColumnWidth(1, 20 * 256);
            sheet.SetColumnWidth(2, 20 * 256);
            sheet.SetColumnWidth(5, 20 * 256);
            ICellStyle style = workbook.CreateCellStyle();
            style.FillPattern = FillPattern.SolidForeground;
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = IndexedColors.Black.Index;
            style.BorderLeft = BorderStyle.Thin;
            style.LeftBorderColor = IndexedColors.Black.Index;
            style.BorderRight = BorderStyle.Thin;
            style.RightBorderColor = IndexedColors.Black.Index;
            style.BorderTop = BorderStyle.Thin;
            style.TopBorderColor = IndexedColors.Black.Index;
            style.Alignment = HorizontalAlignment.Left;
            style.VerticalAlignment = VerticalAlignment.Center;
            IRow headerRow = sheet.CreateRow(0);
            headerRow.CreateCell(0).SetCellValue("配方名称");
            headerRow.CreateCell(1).SetCellValue("门店名称");
            headerRow.CreateCell(2).SetCellValue("设备名称");
            headerRow.CreateCell(3).SetCellValue("配方模式");
            headerRow.CreateCell(4).SetCellValue("执行状态");
            headerRow.CreateCell(5).SetCellValue("使用时间");
            headerRow.Height = 500;
            for (int col = 0; col < 6; col++)
            {
                headerRow.GetCell(col).CellStyle = style;
            }
            for (int i = 0; i < data.Count; i++)
            {
                IRow dataRow = sheet.CreateRow(i + 1);
                dataRow.CreateCell(0).SetCellValue(data[i].RecipeName);
                dataRow.CreateCell(1).SetCellValue(data[i].StoreName);
                dataRow.CreateCell(2).SetCellValue(data[i].DeviceName);
                dataRow.CreateCell(3).SetCellValue(data[i].IsWeight ? "计量模式" : "标准模式");
                dataRow.CreateCell(4).SetCellValue(StatusDesc(data[i].Status));
                dataRow.CreateCell(5).SetCellValue(data[i].UseDate.ToString("yyyy-MM-dd HH:mm:ss"));
                for (int col = 0; col < 6; col++)
                {
                    dataRow.GetCell(col).CellStyle = style;
                }
            }
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ReportData");
            if (Directory.Exists(tempFolder))
                Directory.Delete(tempFolder, true);
            Directory.CreateDirectory(tempFolder);
            var filePath = Path.Combine(tempFolder, $"{Guid.NewGuid()}.xlsx");
            using var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write);
            workbook.Write(stream);
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(filePath));
            return fileBase64;
        }

        /// <summary>
        /// 配方使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<RecipeUseSummaryDto>> GetRecipeUseSummary(RecipeQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrWhiteSpace(dto.RecipeName))
                conModels.Add(new ConditionalModel() { FieldName = "RecipeName", ConditionalType = ConditionalType.Like, FieldValue = dto.RecipeName });
            if (!string.IsNullOrWhiteSpace(dto.StoreName))
                conModels.Add(new ConditionalModel() { FieldName = "StoreName", ConditionalType = ConditionalType.Like, FieldValue = dto.StoreName });
            if (!string.IsNullOrWhiteSpace(dto.DeviceName))
                conModels.Add(new ConditionalModel() { FieldName = "DeviceName", ConditionalType = ConditionalType.Like, FieldValue = dto.DeviceName });
            if (dto.StartTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.GreaterThanOrEqual, FieldValue = dto.StartTime.ToString() });
            if (dto.EndTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.LessThanOrEqual, FieldValue = dto.EndTime.ToString() });
            var list = await _db.Queryable<BPA_RecipeUseDetail>().Where(conModels)
                .WhereIF(dto.IsWeight != null, x => x.IsWeight == dto.IsWeight)
                .WhereIF(dto.Status != null, x => x.Status == dto.Status)
                .Select(a => new RecipeUseSummaryDto
                {
                    Id = a.Id,
                    RecipeName = $"{a.RecipeName}({a.AttributeName})",
                    StoreName = a.StoreName,
                    DeviceName = a.DeviceName,
                    IsWeight = a.IsWeight,
                    Status = a.Status,
                }).ToListAsync();
            var groupList = list.GroupBy(t => (t.RecipeName, t.StoreName, t.DeviceName, t.IsWeight, t.Status));
            var res = (from item in groupList
                       select new RecipeUseSummaryDto
                       {
                           Id = Guid.NewGuid().ToString(),
                           RecipeName = item.Key.RecipeName,
                           StoreName = item.Key.StoreName,
                           DeviceName = item.Key.DeviceName,
                           IsWeight = item.Key.IsWeight,
                           Status = item.Key.Status,
                           UseCount = item.Count()
                       }).OrderByDescending(t => t.UseCount).ToList();
            return res;
        }

        public async Task<string> DownRecipeUseSummary(RecipeQueryDto dto)
        {
            var data = await GetRecipeUseSummary(dto);
            IWorkbook workbook = new XSSFWorkbook();
            ISheet sheet = workbook.CreateSheet("配方使用汇总");
            sheet.SetColumnWidth(0, 20 * 256);
            sheet.SetColumnWidth(1, 20 * 256);
            sheet.SetColumnWidth(2, 20 * 256);
            ICellStyle style = workbook.CreateCellStyle();
            style.FillPattern = FillPattern.SolidForeground;
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = IndexedColors.Black.Index;
            style.BorderLeft = BorderStyle.Thin;
            style.LeftBorderColor = IndexedColors.Black.Index;
            style.BorderRight = BorderStyle.Thin;
            style.RightBorderColor = IndexedColors.Black.Index;
            style.BorderTop = BorderStyle.Thin;
            style.TopBorderColor = IndexedColors.Black.Index;
            style.Alignment = HorizontalAlignment.Left;
            style.VerticalAlignment = VerticalAlignment.Center;
            IRow headerRow = sheet.CreateRow(0);
            headerRow.CreateCell(0).SetCellValue("配方名称");
            headerRow.CreateCell(1).SetCellValue("门店名称");
            headerRow.CreateCell(2).SetCellValue("设备名称");
            headerRow.CreateCell(3).SetCellValue("配方模式");
            headerRow.CreateCell(4).SetCellValue("执行状态");
            headerRow.CreateCell(5).SetCellValue("使用数量");
            headerRow.Height = 500;
            for (int col = 0; col < 6; col++)
            {
                headerRow.GetCell(col).CellStyle = style;
            }
            for (int i = 0; i < data.Count; i++)
            {
                IRow dataRow = sheet.CreateRow(i + 1);
                dataRow.CreateCell(0).SetCellValue(data[i].RecipeName);
                dataRow.CreateCell(1).SetCellValue(data[i].StoreName);
                dataRow.CreateCell(2).SetCellValue(data[i].DeviceName);
                dataRow.CreateCell(3).SetCellValue(data[i].IsWeight ? "计量模式" : "标准模式");
                dataRow.CreateCell(4).SetCellValue(StatusDesc(data[i].Status));
                dataRow.CreateCell(5).SetCellValue(data[i].UseCount);
                for (int col = 0; col < 6; col++)
                {
                    dataRow.GetCell(col).CellStyle = style;
                }
            }
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ReportData");
            if (Directory.Exists(tempFolder))
                Directory.Delete(tempFolder, true);
            Directory.CreateDirectory(tempFolder);
            var filePath = Path.Combine(tempFolder, $"{Guid.NewGuid()}.xlsx");
            using var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write);
            workbook.Write(stream);
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(filePath));
            return fileBase64;
        }

        /// <summary>
        /// 物料使用汇总
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<RecipeBatchingUseSummaryDto>> GetBatchingUseSummary(BatchingQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrWhiteSpace(dto.BatchingName))
                conModels.Add(new ConditionalModel() { FieldName = "BatchingName", ConditionalType = ConditionalType.Like, FieldValue = dto.BatchingName });
            if (!string.IsNullOrWhiteSpace(dto.StoreName))
                conModels.Add(new ConditionalModel() { FieldName = "StoreName", ConditionalType = ConditionalType.Like, FieldValue = dto.StoreName });
            if (!string.IsNullOrWhiteSpace(dto.DeviceName))
                conModels.Add(new ConditionalModel() { FieldName = "DeviceName", ConditionalType = ConditionalType.Like, FieldValue = dto.DeviceName });
            if (dto.StartTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.GreaterThanOrEqual, FieldValue = dto.StartTime.ToString() });
            if (dto.EndTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.LessThanOrEqual, FieldValue = dto.EndTime.ToString() });
            var list = await _db.Queryable<BPA_RecipeBathingUseDetail>().Where(conModels)
                .WhereIF(dto.IsWeight != null, x => x.IsWeight == dto.IsWeight)
                .Select(a => new RecipeBatchingUseSummaryDto
                {
                    Id = a.Id,
                    BatchingName = a.BatchingName,
                    StoreName = a.StoreName,
                    DeviceName = a.DeviceName,
                    UseWeight = a.UseWeight,
                    IsWeight = a.IsWeight
                }).ToListAsync();
            var groupList = list.GroupBy(t => (t.BatchingName, t.StoreName, t.DeviceName, t.IsWeight));
            var res = (from item in groupList
                       select new RecipeBatchingUseSummaryDto
                       {
                           Id = Guid.NewGuid().ToString(),
                           BatchingName = item.Key.BatchingName,
                           StoreName = item.Key.StoreName,
                           DeviceName = item.Key.DeviceName,
                           IsWeight = item.Key.IsWeight,
                           UseWeight = item.Sum(t => t.UseWeight)
                       }).OrderByDescending(t => t.UseWeight).ToList();
            return res;
        }

        public async Task<string> DownBatchingUseSummary(BatchingQueryDto dto)
        {
            var data = await GetBatchingUseSummary(dto);
            IWorkbook workbook = new XSSFWorkbook();
            ISheet sheet = workbook.CreateSheet("物料使用汇总");
            sheet.SetColumnWidth(0, 20 * 256);
            sheet.SetColumnWidth(1, 20 * 256);
            sheet.SetColumnWidth(2, 20 * 256);
            sheet.SetColumnWidth(4, 12 * 256);
            ICellStyle style = workbook.CreateCellStyle();
            style.FillPattern = FillPattern.SolidForeground;
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = IndexedColors.Black.Index;
            style.BorderLeft = BorderStyle.Thin;
            style.LeftBorderColor = IndexedColors.Black.Index;
            style.BorderRight = BorderStyle.Thin;
            style.RightBorderColor = IndexedColors.Black.Index;
            style.BorderTop = BorderStyle.Thin;
            style.TopBorderColor = IndexedColors.Black.Index;
            style.Alignment = HorizontalAlignment.Left;
            style.VerticalAlignment = VerticalAlignment.Center;
            IRow headerRow = sheet.CreateRow(0);
            headerRow.CreateCell(0).SetCellValue("物料名称");
            headerRow.CreateCell(1).SetCellValue("门店名称");
            headerRow.CreateCell(2).SetCellValue("设备名称");
            headerRow.CreateCell(3).SetCellValue("配方模式");
            headerRow.CreateCell(4).SetCellValue("使用重量(g)");
            headerRow.Height = 500;
            for (int col = 0; col < 5; col++)
            {
                headerRow.GetCell(col).CellStyle = style;
            }
            for (int i = 0; i < data.Count; i++)
            {
                IRow dataRow = sheet.CreateRow(i + 1);
                dataRow.CreateCell(0).SetCellValue(data[i].BatchingName);
                dataRow.CreateCell(1).SetCellValue(data[i].StoreName);
                dataRow.CreateCell(2).SetCellValue(data[i].DeviceName);
                dataRow.CreateCell(3).SetCellValue(data[i].IsWeight ? "计量模式" : "标准模式");
                dataRow.CreateCell(4).SetCellValue(data[i].UseWeight.ToString());
                for (int col = 0; col < 5; col++)
                {
                    dataRow.GetCell(col).CellStyle = style;
                }
            }
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "ReportData");
            if (Directory.Exists(tempFolder))
                Directory.Delete(tempFolder, true);
            Directory.CreateDirectory(tempFolder);
            var filePath = Path.Combine(tempFolder, $"{Guid.NewGuid()}.xlsx");
            using var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write);
            workbook.Write(stream);
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(filePath));
            return fileBase64;
        }

        public async Task<ListDto> GetSearchList()
        {
            var result = new ListDto { StoreList = new List<ValueDto>(), DeviceList = new List<ValueDto>() };
            result.StoreList = await _db.Queryable<BPA_RecipeUseDetail>().Select(t => new ValueDto { Value = t.StoreName, Label = t.StoreName }).Distinct().ToListAsync();
            result.DeviceList = await _db.Queryable<BPA_RecipeUseDetail>().Select(t => new ValueDto { Value = t.DeviceName, Label = t.DeviceName }).Distinct().ToListAsync();
            return result;
        }

        public async Task<RankDto> GetMainReport(MainReportQueryDto dto)
        {
            var result = new RankDto();
            var conModels = new List<IConditionalModel>
            {
                new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.GreaterThanOrEqual, FieldValue = dto.StartTime.ToString() },
                new ConditionalModel() { FieldName = "UseDate", ConditionalType = ConditionalType.LessThanOrEqual, FieldValue = dto.EndTime.ToString() }
            };
            if (!string.IsNullOrWhiteSpace(dto.StoreName))
                conModels.Add(new ConditionalModel() { FieldName = "StoreName", ConditionalType = ConditionalType.Equal, FieldValue = dto.StoreName });
            if (!string.IsNullOrWhiteSpace(dto.DeviceName))
                conModels.Add(new ConditionalModel() { FieldName = "DeviceName", ConditionalType = ConditionalType.Equal, FieldValue = dto.DeviceName });
            var recipeList = await _db.Queryable<BPA_RecipeUseDetail>().Where(conModels).ToListAsync();
            var recipeGroup = recipeList.GroupBy(t => t.RecipeName).OrderByDescending(t => t.Count()).Take(10);
            result.RecipeUseBar = new RecipeUseBarDto
            {
                Names = recipeGroup.Select(g => g.Key).ToArray(),
                Values = recipeGroup.Select(g => g.Count()).ToArray()
            };
            if (dto.Grain == "year")
            {
                var recipeYearGroup = recipeList.GroupBy(t => new { t.RecipeName, t.UseDate.Year }).OrderByDescending(t => t.Key.Year);
                var minYear = dto.StartTime.Year;
                var maxYear = dto.EndTime.Year;
                var allYears = Enumerable.Range(minYear, maxYear - minYear + 1).Select(year => year.ToString()).ToArray(); 
                var recipeUseLineDto = new RecipeUseLineDto
                {
                    Xcoos = allYears,
                    Names = recipeYearGroup.Select(g => g.Key.RecipeName).Distinct().ToArray(),
                    Values = new List<LineDto>()
                };
                foreach (var r in recipeYearGroup.GroupBy(g => g.Key.RecipeName))
                {
                    var lineDto = new LineDto { Name = r.Key };
                    int[] data = new int[recipeUseLineDto.Xcoos.Length];
                    for (int i = 0; i < recipeUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.Year.ToString() == recipeUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Count();
                        }
                    }
                    lineDto.Data = data;
                    recipeUseLineDto.Values.Add(lineDto);
                }
                recipeUseLineDto.Xcoos = recipeUseLineDto.Xcoos.Select(x => x + "年").ToArray();
                result.RecipeUseLine = recipeUseLineDto;
            }
            else if (dto.Grain == "month")
            {
                var recipeMonthGroup = recipeList.GroupBy(t => new { t.RecipeName, YearMonth = t.UseDate.ToString("yyyy-MM") }).OrderByDescending(t => t.Key.YearMonth);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allYearMonths = new List<string>();
                for (var date = new DateTime(minDate.Year, minDate.Month, 1); date <= new DateTime(maxDate.Year, maxDate.Month, 1); date = date.AddMonths(1))
                {
                    allYearMonths.Add(date.ToString("yyyy-MM"));
                }
                var recipeUseLineDto = new RecipeUseLineDto
                {
                    Xcoos = allYearMonths.OrderBy(m => m).ToArray(),
                    Names = recipeMonthGroup.Select(g => g.Key.RecipeName).Distinct().ToArray(),
                    Values = new List<LineDto>()
                };
                foreach (var r in recipeMonthGroup.GroupBy(g => g.Key.RecipeName))
                {
                    var lineDto = new LineDto { Name = r.Key };
                    int[] data = new int[recipeUseLineDto.Xcoos.Length];
                    for (int i = 0; i < recipeUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.YearMonth == recipeUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Count();
                        }
                    }
                    lineDto.Data = data;
                    recipeUseLineDto.Values.Add(lineDto);
                }
                recipeUseLineDto.Xcoos = recipeUseLineDto.Xcoos.Select(x =>
                {
                    var parts = x.Split('-');
                    return $"{parts[0]}-{parts[1]}月";
                }).ToArray();
                result.RecipeUseLine = recipeUseLineDto;
            }
            else if (dto.Grain == "week")
            {
                var recipeWeekGroup = recipeList.GroupBy(t => new { t.RecipeName, YearWeek = GetWeekOfYear(t.UseDate) }).OrderByDescending(t => t.Key.YearWeek);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allYearWeeks = new List<string>();
                var startWeek = GetWeekOfYear(minDate);
                var endWeek = GetWeekOfYear(maxDate);
                for (var date = minDate; date <= maxDate; date = date.AddDays(7))
                {
                    var yearWeek = GetWeekOfYear(date);
                    if (!allYearWeeks.Contains(yearWeek))
                        allYearWeeks.Add(yearWeek);
                }
                var recipeUseLineDto = new RecipeUseLineDto
                {
                    Xcoos = allYearWeeks.OrderBy(w => w).ToArray(),
                    Names = recipeWeekGroup.Select(g => g.Key.RecipeName).Distinct().ToArray(),
                    Values = new List<LineDto>()
                };
                foreach (var r in recipeWeekGroup.GroupBy(g => g.Key.RecipeName))
                {
                    var lineDto = new LineDto { Name = r.Key };
                    int[] data = new int[recipeUseLineDto.Xcoos.Length];
                    for (int i = 0; i < recipeUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.YearWeek == recipeUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Count();
                        }
                    }
                    lineDto.Data = data;
                    recipeUseLineDto.Values.Add(lineDto);
                }
                result.RecipeUseLine = recipeUseLineDto;
            }
            else if (dto.Grain == "day")
            {
                var recipeDayGroup = recipeList.GroupBy(t => new { t.RecipeName, Day = t.UseDate.ToString("yyyy-MM-dd") }).OrderByDescending(t => t.Key.Day);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allDays = Enumerable.Range(0, (maxDate - minDate).Days + 1).Select(offset => minDate.AddDays(offset)).Select(date => date.ToString("yyyy-MM-dd")).ToArray();
                var recipeUseLineDto = new RecipeUseLineDto
                {
                    Xcoos = allDays,
                    Names = recipeDayGroup.Select(g => g.Key.RecipeName).Distinct().ToArray(),
                    Values = new List<LineDto>()
                };
                foreach (var r in recipeDayGroup.GroupBy(g => g.Key.RecipeName))
                {
                    var lineDto = new LineDto { Name = r.Key };
                    int[] data = new int[recipeUseLineDto.Xcoos.Length];
                    for (int i = 0; i < recipeUseLineDto.Xcoos.Length; i++)
                    {
                        data[i] = 0;
                        foreach (var a in r.Where(t => t.Key.Day == recipeUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Count();
                        }
                    }
                    lineDto.Data = data;
                    recipeUseLineDto.Values.Add(lineDto);
                }
                recipeUseLineDto.Xcoos = recipeUseLineDto.Xcoos.Select(x =>
                {
                    var parts = x.Split('-');
                    return parts[1] + "-" + parts[2] + "日";
                }).ToArray();
                result.RecipeUseLine = recipeUseLineDto;
            }
            var batchingList = await _db.Queryable<BPA_RecipeBathingUseDetail>().Where(conModels).ToListAsync();
            var batchingGroup = batchingList.GroupBy(t => t.BatchingName).OrderByDescending(t => t.Sum(a => a.UseWeight)).Take(10);
            result.BatchingUseBar = new BatchingUseBarDto
            {
                Names = batchingGroup.Select(g => g.Key).ToArray(),
                Values = batchingGroup.Select(g => g.Sum(a => a.UseWeight)).ToArray()
            };
            if (dto.Grain == "year")
            {
                var batchingYearGroup = batchingList.GroupBy(t => new { t.BatchingName, t.UseDate.Year }).OrderByDescending(t => t.Key.Year);
                var minYear = dto.StartTime.Year;
                var maxYear = dto.EndTime.Year;
                var allYears = Enumerable.Range(minYear, maxYear - minYear + 1).Select(year => year.ToString()).ToArray();
                var batchingUseLineDto = new BatchingUseLineDto
                {
                    Xcoos = allYears,
                    Names = batchingYearGroup.Select(g => g.Key.BatchingName).Distinct().ToArray(),
                    Values = new List<BatchingLineDto>()
                };
                foreach (var r in batchingYearGroup.GroupBy(g => g.Key.BatchingName))
                {
                    var lineDto = new BatchingLineDto { Name = r.Key };
                    var data = new decimal[batchingUseLineDto.Xcoos.Length];
                    for (int i = 0; i < batchingUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.Year.ToString() == batchingUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Sum(t => t.UseWeight);
                        }
                    }
                    lineDto.Data = data;
                    batchingUseLineDto.Values.Add(lineDto);
                }
                batchingUseLineDto.Xcoos = batchingUseLineDto.Xcoos.Select(x => x + "年").ToArray();
                result.BatchingUseLine = batchingUseLineDto;
            }
            else if (dto.Grain == "month")
            {
                var batchingMonthGroup = batchingList.GroupBy(t => new { t.BatchingName, YearMonth = t.UseDate.ToString("yyyy-MM") }).OrderByDescending(t => t.Key.YearMonth);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allYearMonths = new List<string>();
                for (var date = new DateTime(minDate.Year, minDate.Month, 1); date <= new DateTime(maxDate.Year, maxDate.Month, 1); date = date.AddMonths(1))
                {
                    allYearMonths.Add(date.ToString("yyyy-MM"));
                }
                var batchingUseLineDto = new BatchingUseLineDto
                {
                    Xcoos = allYearMonths.OrderBy(m => m).ToArray(),
                    Names = batchingMonthGroup.Select(g => g.Key.BatchingName).Distinct().ToArray(),
                    Values = new List<BatchingLineDto>()
                };
                foreach (var r in batchingMonthGroup.GroupBy(g => g.Key.BatchingName))
                {
                    var lineDto = new BatchingLineDto { Name = r.Key };
                    var data = new decimal[batchingUseLineDto.Xcoos.Length];
                    for (int i = 0; i < batchingUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.YearMonth == batchingUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Sum(t => t.UseWeight);
                        }
                    }
                    lineDto.Data = data;
                    batchingUseLineDto.Values.Add(lineDto);
                }
                batchingUseLineDto.Xcoos = batchingUseLineDto.Xcoos.Select(x =>
                {
                    var parts = x.Split('-');
                    return $"{parts[1]}月";
                }).ToArray();
                result.BatchingUseLine = batchingUseLineDto;
            }
            else if (dto.Grain == "week")
            {
                var batchingWeekGroup = batchingList.GroupBy(t => new { t.BatchingName, YearWeek = GetWeekOfYear(t.UseDate) }).OrderByDescending(t => t.Key.YearWeek);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allYearWeeks = new List<string>();
                var startWeek = GetWeekOfYear(minDate);
                var endWeek = GetWeekOfYear(maxDate);
                for (var date = minDate; date <= maxDate; date = date.AddDays(7))
                {
                    var yearWeek = GetWeekOfYear(date);
                    if (!allYearWeeks.Contains(yearWeek))
                        allYearWeeks.Add(yearWeek);
                }
                var batchingUseLineDto = new BatchingUseLineDto
                {
                    Xcoos = allYearWeeks.OrderBy(w => w).ToArray(),
                    Names = batchingWeekGroup.Select(g => g.Key.BatchingName).Distinct().ToArray(),
                    Values = new List<BatchingLineDto>()
                };
                foreach (var r in batchingWeekGroup.GroupBy(g => g.Key.BatchingName))
                {
                    var lineDto = new BatchingLineDto { Name = r.Key };
                    var data = new decimal[batchingUseLineDto.Xcoos.Length];
                    for (int i = 0; i < batchingUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.YearWeek == batchingUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Sum(t => t.UseWeight);
                        }
                    }
                    lineDto.Data = data;
                    batchingUseLineDto.Values.Add(lineDto);
                }
                result.BatchingUseLine = batchingUseLineDto;
            }
            else if (dto.Grain == "day")
            {
                var batchingDayGroup = batchingList.GroupBy(t => new { t.BatchingName, Day = t.UseDate.ToString("yyyy-MM-dd") }).OrderByDescending(t => t.Key.Day);
                var minDate = dto.StartTime;
                var maxDate = dto.EndTime;
                var allDays = Enumerable.Range(0, (maxDate - minDate).Days + 1).Select(offset => minDate.AddDays(offset)).Select(date => date.ToString("yyyy-MM-dd")).ToArray();
                var batchingUseLineDto = new BatchingUseLineDto
                {
                    Xcoos = allDays,
                    Names = batchingDayGroup.Select(g => g.Key.BatchingName).Distinct().ToArray(),
                    Values = new List<BatchingLineDto>()
                };
                foreach (var r in batchingDayGroup.GroupBy(g => g.Key.BatchingName))
                {
                    var lineDto = new BatchingLineDto { Name = r.Key };
                    var data = new decimal[batchingUseLineDto.Xcoos.Length];
                    for (int i = 0; i < batchingUseLineDto.Xcoos.Length; i++)
                    {
                        foreach (var a in r.Where(t => t.Key.Day == batchingUseLineDto.Xcoos[i]))
                        {
                            data[i] = a.Sum(t => t.UseWeight);
                        }
                    }
                    lineDto.Data = data;
                    batchingUseLineDto.Values.Add(lineDto);
                }
                batchingUseLineDto.Xcoos = batchingUseLineDto.Xcoos.Select(x =>
                {
                    var parts = x.Split('-');
                    return parts[1] + "-" + parts[2] + "日";
                }).ToArray();
                result.BatchingUseLine = batchingUseLineDto;
            }
            var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            var reportSearchLog = await _db.Queryable<BPA_RecipeReportSearchLog>().FirstAsync(t => t.UserId == userId);
            if(reportSearchLog == null)
            {
                reportSearchLog = new BPA_RecipeReportSearchLog
                {
                    UserId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value,
                    SearchLog = JsonConvert.SerializeObject(dto)
                };
                await _db.Insertable(reportSearchLog).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
            }
            else
            {
                reportSearchLog.SearchLog = JsonConvert.SerializeObject(dto);
                await _db.Updateable(reportSearchLog).ExecuteCommandAsync();
            }
            return result;
        }

        private static string StatusDesc(int input)
        {
            switch (input)
            {
                case 1:
                    return "完成";
                case 2:
                    return "强行终止";
                case 3:
                    return "异常";
                default:
                    return "";
            }
        }

        private string GetWeekOfYear(DateTime date)
        {
            var dfi = DateTimeFormatInfo.CurrentInfo;
            var calendar = dfi.Calendar;
            int weekOfYear = calendar.GetWeekOfYear(date, dfi.CalendarWeekRule, dfi.FirstDayOfWeek);
            return $"{date.Year}-{weekOfYear:D2}周";
        }

        public async Task<MainReportQueryDto> GetReportSearchLog()
        {
            var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            var result = new MainReportQueryDto();
            var reportSearchLog = await _db.Queryable<BPA_RecipeReportSearchLog>().FirstAsync(t => t.UserId == userId);
            if(reportSearchLog != null)
                result =  JsonConvert.DeserializeObject<MainReportQueryDto>(reportSearchLog.SearchLog);
            return result;
        }
    }
}
