﻿using BPA.Aliyun.OSS;
using BPA.SAAS.Manage.Application.DataBase.Dtos.ActivationInfo;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Store;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.ActivatioInfo;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Org;
using BPA.SAAS.Manage.Core.Recipe;
using BPA.SAAS.Manage.Core.Store;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class StoreService : IStoreService, ITransient
    {
        private readonly ISqlSugarClient _db;
        private AliyunOssClient _aliyunOssClient;
        public StoreService(ISqlSugarClient db, AliyunOssClient aliyunOssClient)
        {
            _db = db;
            _aliyunOssClient = aliyunOssClient;
        }

        /// <summary>
        /// 门店分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetStorePage(StoreQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_Store, BPA_Organize>((a, b) => new JoinQueryInfos(JoinType.Left, a.OrgId == b.Id))
                .Where(conModels)
                .OrderByDescending(a => a.CreateAt)
                .Select((a, b) => new StoreInfoDto
                {
                    Id = a.Id,
                    Name = a.Name,
                    OrgId = a.OrgId,
                    OrgName = b.Name,
                    Header = a.Header,
                    Phone = a.Phone,
                    Position = a.Position,
                    Description = a.Description,
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil
            {
                Total = total,
                Data = res
            };
        }

        /// <summary>
        /// 获取所有门店
        /// </summary>
        /// <returns></returns>
        public async Task<List<StoreInfoDto>> GetAllStoreList()
        {
            return await _db.Queryable<BPA_Store>().OrderByDescending(t => t.CreateAt).Select(a => new StoreInfoDto
            {
                Id = a.Id,
                Name = a.Name,
                OrgId = a.OrgId,
                Header = a.Header,
                Phone = a.Phone,
                Position = a.Position,
                Description = a.Description
            }).ToListAsync();
        }

        /// <summary>
        /// 更新门店信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateStore(StoreInfoDto dto)
        {
            try
            {
                var res = 0;
                var storeList = await _db.Queryable<BPA_Store>().ToListAsync();
                if (dto.Id == null)
                {
                    var store = storeList.FirstOrDefault(t => t.Name == dto.Name);
                    if (store != null)
                        throw Oops.Oh($"{dto.Name}已存在");
                    store = new BPA_Store
                    {
                        Name = dto.Name,
                        OrgId = dto.OrgId,
                        Header = dto.Header,
                        Phone = dto.Phone,
                        Position = dto.Position,
                        Description = dto.Description,
                    };
                    res = await _db.Insertable(store).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
                else
                {
                    var store = storeList.FirstOrDefault(t => t.Id == dto.Id);
                    if (store == null)
                        throw Oops.Oh($"{dto.Name}不存在或已删除");
                    if(store.Name != dto.Name)
                    {
                        var same = storeList.Any(t => t.Name == dto.Name);
                        if (same)
                            throw Oops.Oh($"{dto.Name}已存在");
                    }
                    store.Name = dto.Name;
                    store.OrgId = dto.OrgId;
                    store.Header = dto.Header;
                    store.Phone = dto.Phone;
                    store.Position = dto.Position;
                    store.Description = dto.Description;
                    res = await _db.Updateable(store).ExecuteCommandAsync();
                }
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除门店信息
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteStore(List<string> Ids)
        {
            try
            {
                _db.Ado.BeginTran();
                var storeList = await _db.Queryable<BPA_Store>().Where(t => Ids.Contains(t.Id)).ToListAsync();
                var deviceList = await _db.Queryable<BPA_DeviceInfo>().ToListAsync();
                foreach (var store in storeList)
                {
                    var anyDevice = deviceList.Any(t => t.StoreId == store.Id);
                    if (anyDevice)
                        throw Oops.Oh($"{store.Name}已有设备，请先删除设备");
                }
                var res = await _db.Deleteable(storeList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return res > 0;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 获取所有设备
        /// </summary>
        /// <returns></returns>
        public async Task<List<DeviceInfoDto>> GetAllDeviceList()
        {
            return await _db.Queryable<BPA_DeviceInfo>().OrderByDescending(t => t.CreateAt).Select(a => new DeviceInfoDto
            {
                Id = a.Id,
                Name = a.Name,
                StoreId = a.StoreId,
                IsEnable = a.IsEnable,
                IsActivation = a.IsActivation,
                AutoKey = a.AutoKey,
                Remark = a.Remark
            }).ToListAsync();
        }

        /// <summary>
        /// 设备分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetDevicePage(DeviceQueryDto dto)
        {
            var logList = await _db.Queryable<BPA_ActivationLog>().ClearFilter().ToListAsync();
            var companyList = await _db.Queryable<BPA_Company>().ToListAsync();
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }
            RefAsync<int> total = 0;
            var activationInfoList = await _db.Queryable<BPA_ActivationInfo>().ClearFilter().ToListAsync();
            var res = await _db.Queryable<BPA_DeviceInfo, BPA_Store>((a, b) => new JoinQueryInfos(JoinType.Left, a.StoreId == b.Id))
                .Where(conModels)
                .OrderByDescending(a => a.CreateAt)
                .Select((a, b) => new DeviceInfoDto
                {
                    Id = a.Id,
                    Name = a.Name,
                    StoreId = a.StoreId,
                    StoreName = b.Name,
                    IsEnable = a.IsEnable,
                    IsActivation = a.IsActivation,
                    AutoKey = a.AutoKey,
                    Remark = a.Remark,
                    LogList = new List<ActivationLogDto>()
                }).Mapper(m =>
                {
                    var activationInfo = activationInfoList.FirstOrDefault(t => t.AutoKey == m.AutoKey);
                    m.LogList = logList.Where(t => t.AutoKey == m.AutoKey).OrderByDescending(t => t.ActivationDate).Select(n => new ActivationLogDto
                    {
                        Id = n.Id,
                        ActivationId = n.ActivationId,
                        OnlineKey = n.OnlineKey,
                        OfflineKey = n.OfflineKey,
                        EffectiveDate = activationInfoList.FirstOrDefault(t=>t.Id == n.ActivationId)?.EffectiveDate.ToString("yyyy-MM-dd"),
                        ActivationDate = n.ActivationDate,
                        AutoKey = n.AutoKey,
                        DeviceName = m.Name,
                        Company = companyList.FirstOrDefault(t => t.Id == n.GroupId)?.Name
                    }).ToList();
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            return new PageUtil
            {
                Total = total,
                Data = res
            };
        }

        /// <summary>
        /// 更新设备信息
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateDevice(DeviceInfoDto dto)
        {
            try
            {
                var res = 0;
                var deviceList = await _db.Queryable<BPA_DeviceInfo>().ToListAsync();
                if (dto.Id == null)
                {
                    var device = deviceList.FirstOrDefault(t => t.Name == dto.Name);
                    if (device != null)
                        throw Oops.Oh($"{dto.Name}已存在");
                    device = new BPA_DeviceInfo
                    {
                        Id = Guid.NewGuid().ToString(),
                        Name = dto.Name,
                        StoreId = dto.StoreId,
                        IsEnable = dto.IsEnable,
                        Remark = dto.Remark
                    };
                    res = await _db.Insertable(device).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
                else
                {
                    var device = deviceList.FirstOrDefault(t => t.Id == dto.Id);
                    if (device == null)
                        throw Oops.Oh($"{dto.Name}不存在或已删除");
                    if(device.Name != dto.Name)
                    {
                        var same = deviceList.Any(t => t.Name == dto.Name);
                        if (same)
                            throw Oops.Oh($"{dto.Name}已存在");
                    }
                    device.Name = dto.Name;
                    device.StoreId = dto.StoreId;
                    device.IsEnable = dto.IsEnable;
                    device.Remark = dto.Remark;
                    res = await _db.Updateable(device).ExecuteCommandAsync();
                }
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除设备信息
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDevice(List<string> Ids)
        {
            try
            {
                _db.Ado.BeginTran();
                var devieList = await _db.Queryable<BPA_DeviceInfo>().Where(t => Ids.Contains(t.Id)).ToListAsync();
                var deviceDelList = new List<BPA_DeviceInfo>();
                foreach (var device in devieList)
                {
                    if (device.IsActivation == 1)
                        throw Oops.Oh("删除失败：所选设备已激活");
                    deviceDelList.Add(device);
                }
                await _db.Deleteable(deviceDelList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 分页查询设备日志
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetDeviceLogPage(DeviceLogQueryDto dto)
        {
            var conModels = new List<IConditionalModel>();
            if (!string.IsNullOrWhiteSpace(dto.FileName))
                conModels.Add(new ConditionalModel() { FieldName = "FileName", ConditionalType = ConditionalType.Like, FieldValue = dto.FileName });
            if (!string.IsNullOrWhiteSpace(dto.StoreName))
                conModels.Add(new ConditionalModel() { FieldName = "StoreName", ConditionalType = ConditionalType.Like, FieldValue = dto.StoreName });
            if (!string.IsNullOrWhiteSpace(dto.DeviceName))
                conModels.Add(new ConditionalModel() { FieldName = "DeviceName", ConditionalType = ConditionalType.Like, FieldValue = dto.DeviceName });
            if (dto.StartTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "CreateAt", ConditionalType = ConditionalType.GreaterThanOrEqual, FieldValue = dto.StartTime.ToString() });
            if (dto.EndTime.HasValue)
                conModels.Add(new ConditionalModel() { FieldName = "CreateAt", ConditionalType = ConditionalType.LessThanOrEqual, FieldValue = dto.EndTime.ToString() });
            RefAsync<int> total = 0;
            var companyList = await _db.Queryable<BPA_Company>().ToListAsync();
            var res = await _db.Queryable<BPA_DeviceLog>().Where(conModels)
                .OrderByDescending(t => t.CreateAt)
                .Select(a => new DeviceLogDto
                {
                    Id = a.Id,
                    FileName = a.FileName,
                    FileUrl = a.FileUrl,
                    StoreName = a.StoreName,
                    DeviceName = a.DeviceName,
                    CreateAt = a.CreateAt,
                    GroupId = a.GroupId,
                }).Mapper(i =>
                {
                    var company = companyList.FirstOrDefault(t => t.Id == i.GroupId);
                    if (company != null)
                        i.CompanyName = company.Name;
                }).ToPageListAsync(dto.Current, dto.PageSize, total);
            var util = new PageUtil
            {
                Total = total,
                Data = res
            };
            return util;
        }
    }
}
